import os
import argparse
import requests as re
import time
import zipfile

import asyncio
import aiohttp
import aiofiles
import traceback


def get_resource_list(args):
    """
    개발, QA, 납품 버전에 따라 다른 API 주소
    resVersion, marketCode, client_version 파라미터
    resVersion 은 초기 버전(0)으로 호출해야 현재버전 - 초기버전 사이의 리소스를 받는다
    marketCode 는 일단 APPLE만
    client_version 은 .h 파일에서 얻는다.
    """

    update_check_url_dict = {
         "DEV": "http://dugout.aceproject.co.kr/mplayball/api/resources/IPHONE_RETINA/0?marketCode=%s&version=%s",
         "QA": "http://kbomanager.aceproject.co.kr/mplayball/api/resources/IPHONE_RETINA/0?marketCode=%s&version=%s",
         "IAP": "http://kbomanager.phonegame.co.kr/mplayball/api/resources/IPHONE_RETINA/0?marketCode=%s&version=%s",
         "DIST": "http://kbomanager.phonegame.co.kr/mplayball/api/resources/IPHONE_RETINA/0?marketCode=%s&version=%s"
         }
    client_version = get_client_version(args.market_code)
    update_check_full_url = update_check_url_dict[args.release_version] % (args.market_code, client_version)

    return re.get(update_check_full_url).json()['data']['resList']


def get_client_version(market_code):
    """
    헤더파일을 파싱하여 클라이언트 버전을 구한다.
    """
    with open("../Classes/build.h") as fp:
        lines = fp.readlines()
        lines = [line.strip() for line in lines if "#define" in line]
        for i in range(len(lines)):
            if '"%s"' % market_code in lines[i]:
                return lines[i+1].split('"')[1]

    raise Exception("Can't get client version")


# 비동기 다운로드
async def download(url, file_name):
    retry = 0
    max_try = 5
    while retry < max_try:
        try:
            # http 요청
            async with aiohttp.ClientSession() as session:
                async with session.get(url) as response:
                    # 요청 실패시 재시도
                    if response.status != 200:
                        raise Exception('다운로드 실패')

                    # 디렉토리 생성
                    if not os.path.exists(os.path.dirname(file_name)):
                        try:
                            os.makedirs(os.path.dirname(file_name))
                        except:
                            pass

                    async with aiofiles.open(file_name, 'wb') as fp:
                        print(file_name,'완료')
                        await fp.write(await response.read())
                        break

        except Exception as ex:
            traceback.print_exc()
            retry += 1
            print('%s %s, 재시도 %d/%d' % (url, ex, retry, max_try))
            continue

        if max_try == retry:
            raise Exception(file_name + ' 다운로드 불가')


def download_files(res_list):
    """
    res_list == {'version':,'urlStorage', files:[{ver,filename,filesize,priority}...]}
    쓰레드로 다운로드
    """

    storage_url = res_list['urlStorage']

    res_files = sorted(res_list['files'], key=lambda x: x["filesize"])


    tasks = [download(storage_url + '/' + res_file['ver'] + '/' + res_file['filename'], res_file['filename']) for res_file in res_files]

    division = 5
    for i in range(division):
        asyncio.run(asyncio.wait(tasks[i::division]))


def write_dat(res_list):
    """
    res_list == {'version':,'urlStorage', files:[{ver,filename,filesize,priority}...]}
    파일 쓰기 resUpdateList_IPHONE_RETINA_v6.dat
    1545890106000
    http: // dev - ss.aceproject.co.kr / mplayball
    [wait]
    [fail]
    """
    latest_ver = res_list['version']
    storage_url = res_list['urlStorage']
    with open('resUpdateList_IPHONE_RETINA_v6.dat','w') as fp:
        print(latest_ver, file=fp)
        print(storage_url, file=fp)
        print('[wait]', file=fp)
        print('[fail]', file=fp)


def unzip_files(res_list):
    """
    res_list == {'version':,'urlStorage', files:[{ver,filename,filesize,priority}...]}
    """

    ordered_zip_list = [file['filename'] for file in sorted(res_list['files'], key=lambda x:x['ver'])
                            if file['filename'].endswith('.zip')]

    for filename in ordered_zip_list:
        with zipfile.ZipFile(filename) as zip:
            zip.extractall()
        os.remove(filename)
        print(filename + ' 압축 해제 완료')


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("release_version", help="DEV or QA or IAP or DIST", choices=["DEV", "QA", "IAP", "DIST"])
    parser.add_argument("market_code", help="APPLE or GOOGLE or TSTORE", choices=["APPLE", "GOOGLE", "TSTORE"])
    args = parser.parse_args()

    start_time = time.time()
    # 1. API 호출
    res_list = get_resource_list(args)

    # 2. API 응답에서 얻은 파일 다운로드
    download_files(res_list)

    # 3. dat 파일 생성
    write_dat(res_list)

    # 4. 다운로드 후 압축 파일이면 압축해제 후 파일 삭제
    unzip_files(res_list)

    end_time = time.time()

    print('경과 시간 %d 초' % int(end_time-start_time))


if __name__ == '__main__':
    main()
